from keras.models import load_model
import joblib
import numpy as np
import xarray as xr


def calculate_rgb(data_vis006, data_vis008, data_nir016,
                  data_tir108, data_sunzen):
    """ Generate RGB from VIS006, VIS008, NIR016, IR108 and solar zenith. """

    truefact = np.where((data_sunzen > 0) & (data_sunzen < 88),
                        np.cos(np.radians(data_sunzen)), 1)

    ir = data_tir108
    ir = np.where(ir < 0, 0, ir)

    term = (ir-302.)
    term = np.where(term < 0, 0, term )
    term = term / 30. * 40.

    r = data_nir016/truefact / 100. * 0.5
    r = np.where(r < 0, 0, r)
    r = np.where(r > 1, 1, r)
    r = r*255

    g = data_vis008/truefact / 138. * 0.8
    g = np.where(g < 0, 0, g)
    g = np.where(g > 1, 1, g)
    g = g*255

    b = data_vis006/truefact / 122. * 0.5
    b = np.where(b < 0, 0, b)
    b = np.where(b > 1, 1, b)
    b = b*255

    r = r * 2.5*0.8
    g = g * 2.5*0.8
    b = b * 3.2*0.8

    c1 = 298.
    c2 = 10.

    r_weight = np.arctan((ir - c1) / c2 * np.pi) / np.pi + 0.5
    multi = np.where(g > b, g , b)
    r = r_weight * r + ((1 - r_weight) * multi)

    r = np.where(r > 255, 255, r)
    g = np.where(g > 255, 255, g)
    b = np.where(b > 255, 255, b)

    ir_min = 180.
    ir_max = 310.
    irb = np.where(ir > ir_max, ir_max, ir)
    irb = np.where(ir < ir_min, ir_min, ir)
    irb = ir_max - irb
    irb = (irb * 255.) / (ir_max - ir_min)
    irb =  np.where(irb > 255, 255, irb)
    irb =  np.where(irb < 0, 0, irb)

    r = np.where(data_sunzen >= 88, irb, r)
    g = np.where(data_sunzen >= 88, irb, g)
    b = np.where(data_sunzen >= 88, irb, b)

    weight = (data_sunzen-70)/18.
    r = np.where((data_sunzen > 70) & (data_sunzen < 88),
                 weight*irb + (1-weight)*r, r)
    g = np.where((data_sunzen > 70) & (data_sunzen < 88),
                 weight*irb + (1-weight)*g, g)
    b = np.where((data_sunzen > 70) & (data_sunzen < 88),
                 weight*irb + (1-weight)*b, b)

    rgb_tmp = np.dstack((r,g,b))

    rgb_tmp = rgb_tmp.astype(np.int)
    rgb_tmp = np.where(rgb_tmp > 255, 255, rgb_tmp)
    return rgb_tmp


#####################################################################
#####################################################################

# load model
model = load_model('mymodel.h5')

# load scaler
scaler = joblib.load('myscaler.pkl')

# TODO: load netCDF file with xarray
# ds = ...

# TODO: input variables of your experiment in correct order
input_variables = ['VIS006']

# count number of variables
n_input_variables = len(input_variables)

# get dimension in x and y direction
xdim = ds['lat'].shape[0]
ydim = ds['lat'].shape[1]

# read input_variables and flatten them
data = np.empty((xdim*ydim, n_input_variables))
for i, v in enumerate(input_variables):
    # read flattened measurements into data array
    data[:, i] = ds[v].data.ravel()

    # multiply VIS calibrated channels by 100.
    if v in ['VIS006', 'VIS008', 'IR_016']:
        data[:, i] = data[:, i] * 100.
    
# apply scaler
data = scaler.transform(data)

# run prediction
cloud_probability = np.squeeze(model.predict(data))
# reshape flattened 1D results to 2D array
cloud_probability = cloud_probability.reshape((xdim, ydim))

# TODO get binary cloud mask from predicted cloud probability
#cma = ....

# get a true color RGB of this scene using the above function
rgb = calculate_rgb(ds['VIS006']*100, ds['VIS008']*100, ds['IR_016']*100, 
                    ds['IR_108'], ds['solzen'])

import matplotlib.pyplot as plt

# TODO: Plot the 2D geospatial cloud_probability, cma and rgb into one image 
#       with three subplots. Don't forget the colorbar for the 
#       cloud_probability plot.
